c	program perturlay.f. Calculates the matrix coefficients to solve
c	the linearized tomographic traveltime inversion in transversely
c	isotropic layered media 
c The LAYERS are described by 3 parameters: 2 velocities (parallel and
c perpendicular with respect to the axis of symmetry) and the tilt of
c the axis of symmetry with respect to the vertical (negative
c clockwise).
c The BOUNDARIES are fixed
c
	parameter (nlayers=200,nmaxray=5000)
        namelist /model/nl,vx,vz,gammafix,dip,zborder,dist
	common /modelo/sx,sz,gammafix,x,z
	real sx(nlayers),realtim(nmaxray),calctim(nmaxray),
     *	sz(nlayers),gammafix(nlayers),dip(nlayers+1),zborder(nlayers+1),
     *	deltat(nmaxray), matrix(nmaxray,2*nlayers),
     *	vx(nlayers),vz(nlayers),
     *	x(nlayers+1,nmaxray),z(nlayers+1,nmaxray),time(nlayers+1,nmaxray)

c In files
	open(3,file='model.in',form='formatted',
     *	status='old')
	open(2,file='/scr/reinaldo/times_calc_ani.out',
     *	status='old')
	open(7,file='/scr/reinaldo/anisolayer_dip.out',
     *	form='unformatted',
     *	status='old')
	open(10,file='/scr/reinaldo/straight.times',
     *	status='old')

c Out files
	open(11,file='perturlay.times',form='formatted',
     *  status='unknown')
	open(12,file='/scr/reinaldo/perturlay.matrix',
     *	form='unformatted',
     *  status='unknown')

	read(3,model) 
	write(*,*) 'number of traveltimes ?'
	read(*,*) nt
	write(*,*) 'scale?'
	read(*,*) scale
	do i=1,nl
		sx(i)=scale/(vx(i))
		sz(i)=scale/(vz(i))
	enddo
	call execute(realtim,calctim,deltat,
     *	dip,zborder,nl,nt,time,matrix,scale)
	stop
	end


	subroutine execute(realtim,calctim,deltat,dip,depth,
     *	nl,nt,time,matrix,scale)
	parameter (nlayers=200,nmaxray=5000)
     	real sx(nlayers),sz(nlayers),tilt(nlayers),
     *	dip(nl+1),depth(nl+1),matrix(nt,2*nl),
     *	realtim(nt),calctim(nt),deltat(nt),
     *	x(nlayers+1,nmaxray),z(nlayers+1,nmaxray),time(nl,nt)
	common /modelo/sx,sz,tilt,x,z
c
	call matmul(dip,depth,nl,nt,time,matrix,scale)
	call deltatime(realtim,calctim,deltat,nt)
	return
	end

        subroutine matmul(dip,depth,nl,nt,time,matrix,
     *	scale)
	parameter (nlayers=200,nmaxray=5000)
     	real sx(nlayers),sz(nlayers),tilt(nlayers),dip(nl),
     *	depth(nl),matrix(nt,2*nl),
     *	x(nlayers+1,nmaxray),z(nlayers+1,nmaxray),time(nl,nt)
	common /modelo/sx,sz,tilt,x,z
	pi=acos(-1.)
	pio180=pi/180.
	do i=1,nl
		tilt(i)=tilt(i)*pio180
	enddo
c fills the matrix with zeros
	do i1=1,nt
		do i2=1,2*nl
		matrix(i1,i2)=0.
		enddo
	enddo
	do j=1,nt
		do i=1,nl+1
			x(i,j)=0.
			z(i,j)=0.
		enddo
	enddo
	do j=1,nt
		do i=1,nl
			time(i,j)=0.
		enddo
	enddo

c i --> counts the layers
c j --> counts the rays
1	read(7,end=999) layer_number,interface,iray,try_x,try_z,try_time
	if(try_time.ne.0.) then
		x(interface,iray)=try_x/scale
		z(interface,iray)=try_z/scale
		time(layer_number,iray)=try_time
	else
		x(interface,iray)=try_x/scale
		z(interface,iray)=try_z/scale
	endif	
	goto 1
999	continue


	do 20 j=1,nt
		if(time(1,j).ne.0.)  then
		oneotimei=1./time(1,j)
		singammai=sin(tilt(1))
		cosgammai=cos(tilt(1))
		deltaxi=(x(2,j)-x(1,j))
		deltazi=(z(2,j)-z(1,j))
		sx2i=sx(1)*sx(1)
		sz2i=sz(1)*sz(1)
		fxi=deltaxi*cosgammai+deltazi*singammai
		fzi=-deltazi*cosgammai+deltaxi*singammai
		matrix(j,1     )=sx(1)*(fxi**2)*oneotimei
		matrix(j,1+  nl)=sz(1)*(fzi**2)*oneotimei
		endif
		do 10 i=2,nl
		    if(time(i,j).ne.0.)  then
			oneotimei=1./time(i,j)
			singammai=sin(tilt(i))
			cosgammai=cos(tilt(i))
			deltaxi=(x(i+1,j)-x(i,j))
			deltazi=(z(i+1,j)-z(i,j))
			sx2i=sx(i)*sx(i)
			sz2i=sz(i)*sz(i)
			fxi=deltaxi*cosgammai+deltazi*singammai
			fzi=-deltazi*cosgammai+deltaxi*singammai
			matrix(j,i)     =sx(i)*(fxi**2)*oneotimei
			matrix(j,i+nl)  =sz(i)*(fzi**2)*oneotimei
	    	    endif
10		continue
20	continue

	write(*,*) 'the size of the matrix is ',nt, 2*nl
	write(12) matrix
	write(*,*) 'matrix written' 
	return
	end


	subroutine deltatime(realtim,calctim,deltat,nt)
	real realtim(nt),calctim(nt),deltat(nt)
	read(10,*) realtim
	read(2,*) calctim
	error=0.	
	n_no_captured=0
	kill=1
	do 1 i=1,nt
		if(calctim(i).eq.0.or.realtim(i).eq.0.) then
			n_no_captured=n_no_captured+1
			kill=0
		endif
		deltat(i)=(realtim(i)-calctim(i))*0.001*kill
		kill=1
		error=error+abs(deltat(i))
1	continue
	write(*,*) error/(nt-n_no_captured), 
     *	' # of traveltimes used ',nt-n_no_captured
	write(20,*) error/(nt-n_no_captured)
	write(11,*) deltat
	return
	end

	subroutine readmodel(nx,v,iunit)
        real v(nx)
        read(iunit,*) v
        close(iunit)
        return
        end
