c	program perturlay.f. Calculates the matrix coefficients to solve
c	the linearized tomographic traveltime inversion in 
c	isotropic layered media 
c The LAYERS are described by 2 velocities (parallel and
c perpendicular with respect to the axis of symmetry).
c The BOUNDARIES are fixed
c
	parameter (nlayers=200,nmaxray=5000)
        namelist /model/nl,vx,vz,gammafix,dip,zborder,dist
	common /modelo/sx,x,z
	real sx(nlayers),realtim(nmaxray),calctim(nmaxray),
     *	sz(nlayers),gammafix(nlayers),dip(nlayers+1),zborder(nlayers+1),
     *	deltat(nmaxray), matrix(nmaxray,nlayers),
     *	vx(nlayers),vz(nlayers),
     *	x(nlayers+1,nmaxray),z(nlayers+1,nmaxray),time(nlayers+1,nmaxray)

c In files
	open(3,file='model.in',form='formatted',
     *	status='old')
	open(2,file='/scr/reinaldo/times_calc_ani.out',
     *	status='old')
	open(7,file='/scr/reinaldo/anisolayer_dip.out',
     *	form='unformatted',
     *	status='old')
	open(10,file='/scr/reinaldo/straight.times',
     *	status='old')

c Out files
	open(11,file='perturlay.times',form='formatted',
     *  status='unknown')
	open(12,file='/scr/reinaldo/perturlay.matrix',
     *	form='unformatted',
     *  status='unknown')

	read(3,model) 
	write(*,*) 'number of traveltimes ?'
	read(*,*) nt
	do i=1,nl
		sx(i)=1./(vx(i))
	enddo
	call execute(realtim,calctim,deltat,
     *	nl,nt,time,matrix)
	stop
	end


	subroutine execute(realtim,calctim,deltat,
     *	nl,nt,time,matrix)
	parameter (nlayers=200,nmaxray=5000)
     	real sx(nlayers),
     *	matrix(nt,nl),
     *	realtim(nt),calctim(nt),deltat(nt),
     *	x(nlayers+1,nmaxray),z(nlayers+1,nmaxray),time(nl,nt)
	common /modelo/sx,x,z
c
	call matmul(nl,nt,time,matrix)
	call deltatime(realtim,calctim,deltat,nt)
	return
	end

        subroutine matmul(nl,nt,time,matrix)
	parameter (nlayers=200,nmaxray=5000)
     	real sx(nlayers),
     *	matrix(nt,nl),
     *	x(nlayers+1,nmaxray),z(nlayers+1,nmaxray),time(nl,nt)
	common /modelo/sx,x,z
	pi=acos(-1.)
	pio180=pi/180.
c fills the matrix with zeros
	do i1=1,nt
		do i2=1,nl
		matrix(i1,i2)=0.
		enddo
	enddo
	do j=1,nt
		do i=1,nl+1
			x(i,j)=0.
			z(i,j)=0.
		enddo
	enddo
	do j=1,nt
		do i=1,nl
			time(i,j)=0.
		enddo
	enddo

c i --> counts the layers
c j --> counts the rays
1	read(7,end=999) layer_number,interface,iray,try_x,try_z,try_time
	if(try_time.ne.0.) then
		x(interface,iray)=try_x
		z(interface,iray)=try_z
		time(layer_number,iray)=try_time
	else
		x(interface,iray)=try_x
		z(interface,iray)=try_z
	endif	
	goto 1
999	continue

        do 20 j=1,nt
                if(time(1,j).ne.0.)  then
                oneotimei=1./time(1,j)
                deltaxi=(x(2,j)-x(1,j))
                deltazi=(z(2,j)-z(1,j))
                fxi=deltaxi
                fzi=-deltazi
                matrix(j,1)=sqrt(fxi**2 + fzi**2)
                endif
                do 10 i=2,nl
                    if(time(i,j).ne.0.)  then
                        oneotimei=1./time(i,j)
                        deltaxi=(x(i+1,j)-x(i,j))
                        deltazi=(z(i+1,j)-z(i,j))
			fxi=deltaxi
                        fzi=-deltazi
                	matrix(j,i)=sqrt(fxi**2 + fzi**2)
                    endif
10              continue
20      continue

	write(*,*) 'the size of the matrix is ',nt, nl
	write(12) matrix
	return
	end

	subroutine deltatime(realtim,calctim,deltat,nt)
	real realtim(nt),calctim(nt),deltat(nt)
	read(10,*) realtim
	read(2,*) calctim
	error=0.	
	n_no_captured=0
	kill=1
	do 1 i=1,nt
		if(calctim(i).eq.0.or.realtim(i).eq.0.) then
			n_no_captured=n_no_captured+1
			kill=0
		endif
		deltat(i)=(realtim(i)-calctim(i))*0.001*kill
		kill=1
		error=error+abs(deltat(i))
1	continue
	write(*,*) error/(nt-n_no_captured), 
     *	' # of traveltimes used ',nt-n_no_captured
	write(20,*) error/(nt-n_no_captured)
	write(11,*) deltat
	return
	end

	subroutine readmodel(nx,v,iunit)
        real v(nx)
        read(iunit,*) v
        close(iunit)
        return
        end
